import { AxiosInstance, Method } from 'axios';
import { MagicalClass } from 'js-magic';
import { IRequestData, IRequestResponse, InterceptorCallback, ErrorHandlingStrategy, RequestHandlerConfig, EndpointConfig } from './types/http-request-handler';
/**
 * Generic Request Handler
 * It creates an Axios instance and handles requests within that instance
 * It handles errors depending on a chosen error handling strategy
 */
export declare class HttpRequestHandler implements MagicalClass {
    /**
     * @var requestInstance Provider's instance
     */
    requestInstance: AxiosInstance;
    /**
     * @var timeout Request timeout
     */
    timeout: number;
    /**
     * @var cancellable Response cancellation
     */
    cancellable: boolean;
    /**
     * @var strategy Request timeout
     */
    strategy: ErrorHandlingStrategy;
    /**
     * @var flattenResponse Response flattening
     */
    flattenResponse: boolean;
    /**
     * @var defaultResponse Response flattening
     */
    defaultResponse: any;
    /**
     * @var logger Logger
     */
    protected logger: any;
    /**
     * @var httpRequestErrorService HTTP error service
     */
    protected httpRequestErrorService: any;
    /**
     * @var requestsQueue    Queue of requests
     */
    protected requestsQueue: Map<string, any>;
    /**
     * Creates an instance of HttpRequestHandler
     *
     * @param {string} baseURL              Base URL for all API calls
     * @param {number} timeout              Request timeout
     * @param {string} strategy             Error Handling Strategy
     * @param {string} flattenResponse      Whether to flatten response "data" object within "data" one
     * @param {*} logger                    Instance of Logger Class
     * @param {*} httpRequestErrorService   Instance of Error Service Class
     */
    constructor({ baseURL, timeout, cancellable, strategy, flattenResponse, defaultResponse, logger, onError, ...config }: RequestHandlerConfig);
    /**
     * Get Provider Instance
     *
     * @returns {AxiosInstance} Provider's instance
     */
    getInstance(): AxiosInstance;
    /**
     * Intercept Request
     *
     * @param {*} callback callback to use before request
     * @returns {void}
     */
    interceptRequest(callback: InterceptorCallback): void;
    /**
     * Maps all API requests
     *
     * @param {string} url                  Url
     * @param {*} data                      Payload
     * @param {EndpointConfig} config       Config
     * @throws {Error}                      If request fails
     * @returns {Promise}                   Request response or error info
     */
    __get(prop: string): any;
    /**
     * Prepare Request
     *
     * @param {string} url                  Url
     * @param {*} data                      Payload
     * @param {EndpointConfig} config       Config
     * @throws {Error}                      If request fails
     * @returns {Promise}                   Request response or error info
     */
    prepareRequest(type: Method, url: string, data?: any, config?: EndpointConfig): Promise<IRequestResponse>;
    /**
     * Build request configuration
     *
     * @param {string} method               Request method
     * @param {string} url                  Request url
     * @param {*}      data                 Request data
     * @param {EndpointConfig} config       Request config
     * @returns {AxiosInstance} Provider's instance
     */
    protected buildRequestConfig(method: string, url: string, data: any, config: EndpointConfig): EndpointConfig;
    /**
     * Process global Request Error
     *
     * @param {Error} error      Error instance
     * @returns {AxiosInstance} Provider's instance
     */
    protected processRequestError(error: Error): void;
    /**
     * Output error response depending on chosen strategy
     *
     * @param {Error} error      Error instance
     * @param {EndpointConfig} requestConfig   Per endpoint request config
     * @returns {AxiosInstance} Provider's instance
     */
    protected outputErrorResponse(error: Error, requestConfig: EndpointConfig): Promise<IRequestResponse>;
    /**
     * Output error response depending on chosen strategy
     *
     * @param {Error} error                     Error instance
     * @param {EndpointConfig} requestConfig    Per endpoint request config
     * @returns {*}                             Error response
     */
    isRequestCancelled(error: Error, requestConfig: EndpointConfig): boolean;
    /**
     * Automatically Cancel Previous Requests
     *
     * @param {string} type                    Request type
     * @param {string} url                     Request url
     * @param {EndpointConfig} requestConfig   Per endpoint request config
     * @returns {AxiosInstance} Provider's instance
     */
    protected addCancellationToken(type: string, url: string, requestConfig: EndpointConfig): {
        cancelToken?: undefined;
    } | {
        cancelToken: any;
    };
    /**
     * Handle Request depending on used strategy
     *
     * @param {object} payload                      Payload
     * @param {string} payload.type                 Request type
     * @param {string} payload.url                  Request url
     * @param {*} payload.data                      Request data
     * @param {EndpointConfig} payload.config       Request config
     * @throws {Error}
     * @returns {Promise} Response Data
     */
    protected handleRequest({ type, url, data, config, }: IRequestData): Promise<IRequestResponse>;
    /**
     * Process request response
     *
     * @param response Response object
     * @returns {*} Response data
     */
    protected processResponseData(response: any): any;
}
